==> MBrane-1.0.8.crate: MBrane-1.0.8/src/ts/Vector4D.rs <==
#![allow(non_snake_case)]

use std::ops::{Add, Sub, Mul, Div};

#[derive(Debug)]
pub struct Vector4D {
    pub X: f64,
    pub Y: f64,
    pub Z: f64,
    pub T: f64,
}

pub trait VectorOperations {
    fn Magnitude(&self) -> f64;
    fn Normalize(&self) -> Vector4D;
    fn Dot(&self, rhs: Vector4D) -> f64;
}

impl Vector4D {
    pub fn New(&self, nx: f64, ny: f64, nz: f64, nt: f64) -> Vector4D {
        Vector4D {
            X: nx,
            Y: ny,
            Z: nz,
            T: nt,
        }
    }

    pub fn SelfNormalize(&mut self) {
        let magnitude = self.Magnitude();
        self.X = self.X / magnitude;
        self.Y = self.Y / magnitude;
        self.Z = self.Z / magnitude;
        self.T = self.T / magnitude;
    }
}

impl VectorOperations for Vector4D {
    fn Magnitude(&self) -> f64 {
        (((self.X * self.X) + (self.Y * self.Y) + (self.Z * self.Z) + (self.T * self.T))).sqrt()
    }

    fn Normalize(&self) -> Vector4D {
        let magnitude = self.Magnitude();
        Vector4D::New(self,
                      self.X / magnitude,
                      self.Y / magnitude,
                      self.Z / magnitude,
                      self.T / magnitude)
    }

    fn Dot(&self, rhs: Vector4D) -> f64 {
        (self.X * rhs.X) + (self.Y * rhs.Y) + (self.Z * rhs.Z) + (self.T * rhs.T)
    }
}

impl Add for Vector4D {
    type Output = Vector4D;

    fn add(self, rhs: Vector4D) -> Vector4D {
        Vector4D::New(&self,
                      self.X + rhs.X,
                      self.Y + rhs.Y,
                      self.Z + rhs.Z,
                      self.T + rhs.T)
    }
}

impl Sub for Vector4D {
    type Output = Vector4D;

    fn sub(self, rhs: Vector4D) -> Vector4D {
        Vector4D::New(&self,
                      self.X - rhs.X,
                      self.Y - rhs.Y,
                      self.Z - rhs.Z,
                      self.T - rhs.T)
    }
}

impl Mul for Vector4D {
    type Output = Vector4D;

    fn mul(self, rhs: Vector4D) -> Vector4D {
        Vector4D::New(&self,
                      self.X * rhs.X,
                      self.Y * rhs.Y,
                      self.Z * rhs.Z,
                      self.T * rhs.T)
    }
}

impl Div for Vector4D {
    type Output = Vector4D;

    fn div(self, rhs: Vector4D) -> Vector4D {
        Vector4D::New(&self,
                      self.X / rhs.X,
                      self.Y / rhs.Y,
                      self.Z / rhs.Z,
                      self.T / rhs.T)
    }
}

==> MBrane-1.0.10.crate: MBrane-1.0.10/src/ts/Vector4D.rs <==
#![allow(non_snake_case)]

use std::ops::{Add, Sub, Mul, Div};

#[derive(Debug)]
pub struct Vector4D {
    pub X: f64,
    pub Y: f64,
    pub Z: f64,
    pub T: f64,
}

pub trait VectorOperations {
    fn magnitude(&self) -> f64;
    fn normalize(&self) -> Vector4D;
    fn dot(&self, rhs: Vector4D) -> f64;
}

impl Vector4D {
    pub fn new(nx: f64, ny: f64, nz: f64, nt: f64) -> Vector4D {
        Vector4D {
            X: nx,
            Y: ny,
            Z: nz,
            T: nt,
        }
    }

    pub fn self_normalize(&mut self) {
        let magnitude = self.magnitude();
        self.X = self.X / magnitude;
        self.Y = self.Y / magnitude;
        self.Z = self.Z / magnitude;
        self.T = self.T / magnitude;
    }
}

impl VectorOperations for Vector4D {
    fn magnitude(&self) -> f64 {
        (((self.X * self.X) + (self.Y * self.Y) + (self.Z * self.Z) + (self.T * self.T))).sqrt()
    }

    fn normalize(&self) -> Vector4D {
        let magnitude = self.magnitude();
        Vector4D::new(self.X / magnitude,
                      self.Y / magnitude,
                      self.Z / magnitude,
                      self.T / magnitude)
    }

    fn dot(&self, rhs: Vector4D) -> f64 {
        (self.X * rhs.X) + (self.Y * rhs.Y) + (self.Z * rhs.Z) + (self.T * rhs.T)
    }
}

impl Add for Vector4D {
    type Output = Vector4D;

    fn add(self, rhs: Vector4D) -> Vector4D {
        Vector4D::new(self.X + rhs.X,
                      self.Y + rhs.Y,
                      self.Z + rhs.Z,
                      self.T + rhs.T)
    }
}

impl Sub for Vector4D {
    type Output = Vector4D;

    fn sub(self, rhs: Vector4D) -> Vector4D {
        Vector4D::new(self.X - rhs.X,
                      self.Y - rhs.Y,
                      self.Z - rhs.Z,
                      self.T - rhs.T)
    }
}

impl Mul for Vector4D {
    type Output = Vector4D;

    fn mul(self, rhs: Vector4D) -> Vector4D {
        Vector4D::new(self.X * rhs.X,
                      self.Y * rhs.Y,
                      self.Z * rhs.Z,
                      self.T * rhs.T)
    }
}

impl Div for Vector4D {
    type Output = Vector4D;

    fn div(self, rhs: Vector4D) -> Vector4D {
        Vector4D::new(self.X / rhs.X,
                      self.Y / rhs.Y,
                      self.Z / rhs.Z,
                      self.T / rhs.T)
    }
}
